@@ -560,3 +560,78 @@ def test_stream_similarity_top_category_functions():
     assert all(
         actual_key == expected_key for actual_key, expected_key in zip(actual, expected)
     )
+
+
+def test_user_pwatched_cleanup():
+    user_pwatched_data = {
+        "version": 1,
+        "data": {
+            "sel_thumb": {"attempts": 1, "watched": 1},
+            "ch_swtch": {"attempts": 2, "watched": 0},
+        },
+    }
+    user_pwatched_msg = features_pb2_v1.UserPWatched()
+    ProtoParseDict(js_dict=user_pwatched_data, message=user_pwatched_msg)
+    user = {"PWATCHED#6M": user_pwatched_msg}
+    out = {}
+    utils.user_pwatched_cleanup(
+        user=user, entry_contexts=["autoplay", "sel_thumb", "ch_swtch"], out=out
+    )
+    expected = pd.Series(
+        {
+            "USER_AUTOPLAY_6M_TOTAL_ATTEMPTS": 0,
+            "USER_AUTOPLAY_6M_TOTAL_WATCHED": 0,
+            "USER_SEL_THUMB_6M_TOTAL_ATTEMPTS": 1,
+            "USER_SEL_THUMB_6M_TOTAL_WATCHED": 1,
+            "USER_CH_SWTCH_6M_TOTAL_ATTEMPTS": 2,
+            "USER_CH_SWTCH_6M_TOTAL_WATCHED": 0,
+        }
+    )
+    actual = pd.Series(out).loc[expected.index]
+    assert (expected == actual).all()
+
+
+def test_user_pselect_cleanup():
+    user_pselect_data = {
+        "version": 1,
+        "data": {
+            "all_browsed": {
+                "first_pos": {
+                    "total_selects": 0,
+                    "total_selects_and_watched": 0,
+                    "total_browsed": 1,
+                },
+                "rest_pos": {
+                    "total_selects": 2,
+                    "total_selects_and_watched": 2,
+                    "total_browsed": 1,
+                },
+            },
+            "up_to_4_browsed": {
+                "first_pos": {
+                    "total_selects": 0,
+                    "total_selects_and_watched": 0,
+                    "total_browsed": 1,
+                },
+                "rest_pos": {
+                    "total_selects": 2,
+                    "total_selects_and_watched": 2,
+                    "total_browsed": 0,
+                },
+            },
+        },
+    }
+    user_pselect_msg = features_pb2_v1.UserPSelect()
+    ProtoParseDict(js_dict=user_pselect_data, message=user_pselect_msg)
+    user = {"PSELECT#6M": user_pselect_msg}
+    out = {}
+    utils.user_pselect_cleanup(user=user, position_debiasing="up_to_4_browsed", out=out)
+    expected = pd.Series(
+        {
+            "USER_6M_TOTAL_BROWSED_UP_TO_4_BROWSED": 1,
+            "USER_6M_TOTAL_SELECTS_UP_TO_4_BROWSED": 2,
+            "USER_6M_TOTAL_SELECTS_AND_WATCHED_UP_TO_4_BROWSED": 2,
+        }
+    )
+    actual = pd.Series(out).loc[expected.index]
+    assert (actual == expected).all()
